import React, { ComponentProps } from "react"
import { MinistreamApiClient } from "./api"
import { resolve } from "path"

type MediaContainerProps = {
    selectedStream: string | undefined
    api: MinistreamApiClient
}

type MediaContainerState = {
    iceReady: boolean
    remoteReady: boolean
    videoReady: boolean
}

export class MediaContainer extends React.Component<MediaContainerProps, MediaContainerState> {
    api: MinistreamApiClient
    pc: RTCPeerConnection
    ms: MediaStream

    constructor(props: MediaContainerProps) {
        super(props)

        this.api = props.api
        this.state = {
            iceReady: false,
            remoteReady: false,
            videoReady: false
        }

        this.pc = new RTCPeerConnection({
            iceServers: [{ urls: "stun:stun.l.google.com:19302" }]
        })
        this.pc.addTransceiver("video")
        this.pc.addTransceiver("audio")

        this.ms = new MediaStream()

        this.pc.ontrack = (event) => {
            console.log("Got track(s).")

            event.streams.forEach(st => {
                st.getTracks().forEach(t => {
                    this.ms.addTrack(t)
                })
            })
        }


        this.pc.oniceconnectionstatechange = () => {
            console.log("ICE state changed to " + this.pc?.iceConnectionState)
        }
    }

    async componentDidMount() {
        this.pc.onicecandidate = (event) => {
            if (!event.candidate) {
                console.log("ICE gathering complete.")
                this.setState((state) => {
                    return { iceReady: true, remoteReady: state.remoteReady }
                })
            } else {
                console.log("Adding ICE candidate: " + event.candidate)
            }
        }
        const offer = await this.pc?.createOffer()
        await this.pc?.setLocalDescription(offer)
        console.log("Set local description.")
    }

    async componentDidUpdate() {
        if (!this.state.iceReady) {
            return
        }
        if (!this.props.selectedStream) {
            return
        }
        if (!this.state.remoteReady) {
            console.log("Setting up remote for " + this.props.selectedStream + ".")
            const remote = await this.setupRemote()
            if (remote) {
                this.pc.setRemoteDescription(remote)
                console.log("Set up remote")
                this.setState((state) => {
                    return { iceReady: state.iceReady, remoteReady: true }
                })
            } else {
                console.log("Could not set remote")
            }
        }
    }

    private setupRemote() {
        if (!this.props.selectedStream) {
            return
        }
        const localOfferSdp = this.pc?.localDescription?.sdp
        if (!localOfferSdp) {
            console.log("Could not get local description")
            return
        }
        const localOffer = new RTCSessionDescription({ type: "offer", sdp: localOfferSdp })
        return this.api.postOffer(this.props.selectedStream, localOffer)
    }

    renderVideo() {
        if (!this.props.selectedStream) {
            return <p>Waiting for stream selection.</p>
        }
        if (!this.state.iceReady) {
            return <p>Waiting for ICE gathering to complete.</p>
        }
        if (!this.state.remoteReady) {
            return <p>Waiting for remote.</p>
        }
        return (
            <>
                <p id="loading-text">Waiting for video to load.</p>
                <video ref={el => {
                    if (el && this.ms) {
                        el.srcObject = this.ms
                        el.hidden = true
                        el.addEventListener('loadeddata', () => {
                            const t = document.getElementById("loading-text")
                            el.hidden = false
                            if (t) {
                                t.hidden = true
                            }
                        })
                    }
                }} id="video" autoPlay muted controls />
            </>
        )
    }

    render() {
        return (
            <div id="main">
                <main>
                    <div className="video-wrapper">
                        {this.renderVideo()}
                    </div>
                </main>
            </div>
        )
    }
}
